/*
   	ASMToCHex Qt
    Copyright (C) 2006 Jonas Gehring
        
    ASMToCHex Qt is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    ASMToCHex Qt is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ASMToCHex Qt; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "mainwindow.h"


// Constructor
MainWindow::MainWindow() : QWidget()
{
	setFixedSize(435, 277);
	setWindowIcon(QIcon(QString::fromUtf8(":/icons/images/atoch.png")));
    
	// Input Group
	inputGroup = new QGroupBox(this);
    inputGroup->setGeometry(QRect(10, 10, 411, 51));
	inputGroup->setTitle("Input file");
    inputEdit = new QLineEdit(inputGroup);
    inputEdit->setGeometry(QRect(10, 20, 291, 20));
    inputButton = new QPushButton(inputGroup);
    inputButton->setGeometry(QRect(314, 20, 81, 20));
    inputButton->setIcon(QIcon(QString::fromUtf8(":/icons/images/fileopen.png")));
	inputButton->setText("&Browse...");

	// Output Group
	outputGroup = new QGroupBox(this);
    outputGroup->setGeometry(QRect(10, 70, 411, 51));
	outputGroup->setTitle("Output file");
    outputEdit = new QLineEdit(outputGroup);
    outputEdit->setGeometry(QRect(10, 20, 291, 20));
    outputButton = new QPushButton(outputGroup);
    outputButton->setGeometry(QRect(314, 20, 81, 20));
    outputButton->setIcon(QIcon(QString::fromUtf8(":/icons/images/filesave.png")));
	outputButton->setText("&Browse...");

	// Type Group
    typeGroup = new QGroupBox(this);
    typeGroup->setGeometry(QRect(10, 130, 151, 101));
    typeGroup->setTitle("Data type");
    ucharRadio = new QRadioButton(typeGroup);
    ucharRadio->setGeometry(QRect(10, 20, 101, 20));
    ucharRadio->setChecked(true);
    ucharRadio->setText("unsigned char");
    ushortRadio = new QRadioButton(typeGroup);
    ushortRadio->setGeometry(QRect(10, 45, 101, 20));
    ushortRadio->setText("unsigned short");
    ulongRadio = new QRadioButton(typeGroup);
    ulongRadio->setGeometry(QRect(10, 70, 101, 20));
    ulongRadio->setText("unsigned long");

	// Format Group
    formatGroup = new QGroupBox(this);
    formatGroup->setGeometry(QRect(180, 130, 241, 101));
	formatGroup->setTitle("Array format");
    enableRadio = new QRadioButton(formatGroup);
    enableRadio->setGeometry(QRect(10, 45, 131, 20));
    enableRadio->setChecked(true);
	enableRadio->setText("Enable array format");
    disableRadio = new QRadioButton(formatGroup);
    disableRadio->setGeometry(QRect(10, 20, 131, 20));
	disableRadio->setText("Disable array format");
    arrayEdit = new QLineEdit(formatGroup);
    arrayEdit->setGeometry(QRect(30, 70, 191, 20));
	arrayEdit->setText("arrayname");
	
	// Buttons at the bottom
    convertButton = new QPushButton(this);
    convertButton->setGeometry(QRect(10, 240, 151, 31));
    convertButton->setDefault(true);
	convertButton->setText("&Convert!");
    aboutButton = new QPushButton(this);
    aboutButton->setGeometry(QRect(180, 245, 91, 21));
    aboutButton->setIcon(QIcon(QString::fromUtf8(":/icons/images/qt.png")));
	aboutButton->setText("&About...");
    closeButton = new QPushButton(this);
    closeButton->setGeometry(QRect(325, 245, 96, 21));
	closeButton->setText("Close &Window");

	
	// Connect actions
	connect(inputButton, SIGNAL(clicked()), this, SLOT(inputFile()));
	connect(outputButton, SIGNAL(clicked()), this, SLOT(outputFile()));
	connect(closeButton, SIGNAL(clicked()), this, SLOT(close()));
	connect(disableRadio, SIGNAL(toggled(bool)), arrayEdit, SLOT(setDisabled(bool)));
	connect(convertButton, SIGNAL(clicked()), this, SLOT(convertFile()));
	connect(aboutButton, SIGNAL(clicked()), this, SLOT(showCredits()));
	
	setWindowTitle("ASMToCHex Qt");
}


// Browse input file
void MainWindow::inputFile()
{
	QString fileName = QFileDialog::getOpenFileName(0,
								QObject::tr("Open input file..."),
								inputEdit->text(),
								QObject::tr("Assembly Source Files (*.asm)"));
	if (!fileName.isEmpty())
		inputEdit->setText(fileName);
}

// Browse output file
void MainWindow::outputFile()
{
	QString fileName = QFileDialog::getSaveFileName(0,
								QObject::tr("Save output file..."),
								outputEdit->text(),
								QObject::tr("C Source Files (*.c);;C Header Files (*.h)"));
	if (!fileName.isEmpty())
		outputEdit->setText(fileName);
}

// The program's heart: the converter:
void MainWindow::convertFile()
{
    QFile inFile(inputEdit->text());
    QFile outFile(outputEdit->text());
	QString buffer;
	qint16 index, firstentry = -1, i, n = 0, flag = 0, numel = 0;
	
	// Open files
	if (!inFile.open(QFile::ReadOnly | QFile::Text))
	{
        QMessageBox::critical(this, QObject::tr("ASMToCHex Qt"), QObject::tr("Error opening input file"), 1, QMessageBox::NoButton); 
        return;
    }
	if (!outFile.open(QFile::WriteOnly | QFile::Text))
	{
		inFile.close();
		QMessageBox::critical(this, QObject::tr("ASMToCHex Qt"), QObject::tr("Error opening output file"), 1, QMessageBox::NoButton); 
        return;
	}
	
	QTextStream out(&outFile);
	
	// Sepcify format
	if (ucharRadio->isChecked())
	{
		flag = 1;
	}
	else if (ushortRadio->isChecked())
	{
		flag = 2;
	}
	else if (ulongRadio->isChecked())
	{
		flag = 4;
	}
			
	// Count number of entries
	while ((buffer = inFile.readLine()) != NULL)
	{
		for (i = 0; i < buffer.length(); i++)
		{
			if (buffer[i] == '$')
				numel++;	
		}
	}
		
	numel /= flag;
			
	if (!inFile.seek(0))
	{
		inFile.close();
		if (!inFile.open(QFile::ReadOnly | QFile::Text))
		{
			outFile.close();	
			return;
		}
	}
			
	// Read every single line until the end of the file is reached 
	while ((buffer = inFile.readLine()) != NULL) 
	{
		index = 0;
		
		// Search for specific beginnings and replace them
		if (buffer.indexOf(" dc.b ") == 0)
		{
			index = 6;
		}
		else if (buffer.indexOf(" ;") == 0)
		{
			out << "// ";	
			index = 2;
		}
		else if (buffer.indexOf(";") == 0)
		{
			out << "// ";
			index = 1;
		}
		else if (buffer.indexOf(",") != 0 ||
			buffer.indexOf("$") != 0)
		{
			if (firstentry == -1) 
			{
				if (enableRadio->isChecked())
				{
					if (flag == 1) 
						out << "unsigned char " << arrayEdit->text() << "["
							<< numel << "] =";
					else if (flag == 2) 
						out << "unsigned short " << arrayEdit->text() << "["
							<< numel << "] =";
					else if (flag == 4)
						out << "unsigned long " << arrayEdit->text() << "["
							<< numel << "] =";
					
					out << "\n{\n";
				}					
			}
			firstentry = 0;
			continue;
		}
				
		// Go on with the rest of the characters
		for (i = 0; i < buffer.length()-index; i++)
		{
			// Remove "$"-Prefixes and replace them with "0x"
			if (buffer.data()[index+i] ==  '$') 
			{
				n++;
                        
				if ((n/flag) == 1 || i == 0) 
				{                  
					n = 0; 
					// Write "0x" at the beginning of an entry
					if (firstentry == 0)
					{  
						out << " 0x";
						firstentry++; 
					}
					else
					{
						out << ",0x";
					}								
				}
				continue;                            
  				} 
			else if (buffer.data()[index+i] != ',') 
			{
				// Write data to file
				out << buffer.data()[index+i];
				continue;
			}
		}
		
		// End of a line
		//out << '\n';
	}
			
	// Write "};" at the end if needed
	if (enableRadio->isChecked())
	{
		out << "};\n";
	}
	
	out << "\n// Converted to C Hex with ASMToCHex Qt by Jonas Gehring";
			
	QString result = QString("Data successfull converted.\nNumber of Items: %1").arg(numel);
			
	/* Message that converting was successfull */
	QMessageBox::information(this, "ASMToCHex Qt", result, 1);

	// Close files
	inFile.close();
	outFile.close();
}


// Shows a dialog with the credits of this program
void MainWindow::showCredits()
{
	QMessageBox::about(this, "ASMToCHex Qt", "ADMToCHex Qt\nwritten by Jonas Gehring\nreleased under boolsoft\n\nmade with Qt 4.1.0\nsee http://www.trolltech.com/qt");
}
